//--------------------------------------------------------------
// File     : stm32_ub_mcp25050.c
// Datum    : 21.09.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO, STM32_UB_CAN1
// Funktion : CAN IO-Expander (MCP25050)
//
// Hinweis  : der MCP25050 muss mit dem File
//            "ub01_4MHz_8us.hex" programmiert sein
//
// Setting  : Quarz = 4MHz, CAN-Baud = 125 kHz
//
// MCP25050 : MCP_GP0 [Pin-1]  (Dig_IN, Dig_OUT, Analog_IN)
//            MCP_GP1 [Pin-2]  (Dig_IN, Dig_OUT, Analog_IN)
//            MCP_GP2 [Pin-3]  (Dig_IN, Dig_OUT, Analog_IN, PWM-Out)
//            MCP_GP3 [Pin-4]  (Dig_IN, Dig_OUT, Analog_IN, PWM-Out)
//            MCP_GP4 [Pin-5]  (Dig_IN, Dig_OUT)
//            MCP_GP5 [Pin-6]  (Dig_IN, Dig_OUT)
//            MCP_GP6 [Pin-10] (Dig_IN, Dig_OUT)
//            MCP_GP7 [Pin-11] (Dig_IN)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_mcp25050.h"




//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
CAN1_TX_FRAME_t tx_mcp25050;
CAN1_RX_FRAME_t rx_mcp25050;


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_MCP_delay(uint32_t delay);
void P_MCP_initADC(uint8_t adc_speed);


//--------------------------------------------------------------
// Init vom MCP25050 und check ob IC vorhanden
//
// Return_wert :
//  -> ERROR   , MCP25050 wurde nicht gefunden
//  -> SUCCESS , MCP25050 wurde gefunden
//--------------------------------------------------------------
ErrorStatus UB_MCP25050_Init(void)
{
  ErrorStatus ret_wert=ERROR; 

  // init vom CAN-Bus
  UB_CAN1_Init();

  // test ob MCP25050 am CAN-BUS haengt
  ret_wert=UB_MCP25050_Check();

  return(ret_wert);
}


//--------------------------------------------------------------
// check ob MCP25050 am CAN-BUS haengt
//
// Return_wert :
//  -> ERROR   , MCP25050 wurde nicht gefunden
//  -> SUCCESS , MCP25050 wurde gefunden
//--------------------------------------------------------------
ErrorStatus UB_MCP25050_Check(void)
{
  ErrorStatus ret_wert=ERROR;
  uint32_t n;
  CAN1_STATUS_t check;

  //-------------------------------
  // Config-Register auslesen
  // (5 Datenbytes werden erwartet)
  //-------------------------------
  tx_mcp25050.can_id=(MCP_RXF0|MCP_RD_CONFIG);
  tx_mcp25050.anz_bytes=0x05;
  // STD-Remote-Frame senden
  UB_CAN1_send_std_remote(tx_mcp25050);  

  // warten bis antwort erhalten
  for(n=0;n<MCP_WAIT4RECEIVE;n++) {
    check=UB_CAN1_receive(&rx_mcp25050);
    if(check==CAN1_RX_READY) {
      // wenn etwas empfangen wurde
      // test ob Antwort vom MCP25050
      if(rx_mcp25050.can_id==(MCP_RXF0|MCP_RD_CONFIG)) {
        if(rx_mcp25050.frame_mode==CAN1_STD_DATA) {
          ret_wert=SUCCESS;

          // Defaulteinstellungen (alle Pins auf Digital-Eingang)
          UB_MCP25050_writeRegister(MCP_ADR_ADCON1, 0x0F, 0x0F);
          UB_MCP25050_writeRegister(MCP_ADR_GPDR, 0xFF, 0xFF);

          break;
        }
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// einstellen vom Modus eines Pins
//
// pin  : [MCP_GP0 bis MCP_GP7]
// mode : [MCP_GPIO_OUT, MCP_GPIO_IN, MCP_PWM_OUT, MCP_ADC_IN]
//--------------------------------------------------------------
void UB_MCP25050_PIN_Define(MCP_GPIO_t pin, MCP_PIN_MODE_t mode)
{
  uint8_t pin_maske=0x00;
  uint8_t reg_gpddr=0x00;   // in/out         (Hi=In)
  uint8_t reg_adcon1=0x00;  // digital/analog (Hi=digital)

  // GP7 geht nicht als Ausgang
  if((mode==MCP_GPIO_OUT) && (pin==MCP_GP7)) return;

  // nur GP2 und GP3 gehen als PWM
  if(mode==MCP_PWM_OUT) {
    if((pin!=MCP_GP2) && (pin!=MCP_GP3)) return;
  }

  // GP4-GP7 gehen nicht als ADC
  if(mode==MCP_ADC_IN) {
    if((pin==MCP_GP4) || (pin==MCP_GP5)) return;
    if((pin==MCP_GP6) || (pin==MCP_GP7)) return;
  }

  if(pin==MCP_GP0) pin_maske=0x01;
  if(pin==MCP_GP1) pin_maske=0x02;
  if(pin==MCP_GP2) pin_maske=0x04;
  if(pin==MCP_GP3) pin_maske=0x08;
  if(pin==MCP_GP4) pin_maske=0x10;
  if(pin==MCP_GP5) pin_maske=0x20;
  if(pin==MCP_GP6) pin_maske=0x40;
  if(pin==MCP_GP7) pin_maske=0x80;

  if(mode==MCP_GPIO_OUT) { 
    // GPIO auf Ausgang         
    reg_gpddr=0x00;             // out
    reg_adcon1=0xFF;            // digital
  }
  else if(mode==MCP_PWM_OUT) { 
    // GPIO auf PWM-Ausgang         
    reg_gpddr=0x00;             // out
    reg_adcon1=0xFF;            // digital
  }
  else if(mode==MCP_GPIO_IN) {
    // GPIO auf Eingang
    reg_gpddr=0xFF;             // in
    reg_adcon1=0xFF;            // digital
  }
  else if(mode==MCP_ADC_IN) {
    // GPIO auf ADC-Eingang
    reg_gpddr=0xFF;             // in
    reg_adcon1=0x00;            // analog
  }
  else {
    return; // falscher Mode
  }  
  
  // GPDDR-Register setzen (immer)
  UB_MCP25050_writeRegister(MCP_ADR_GPDR, reg_gpddr, pin_maske);

  // GPLAT-Register setzen (immer auf Lo)
  UB_MCP25050_writeRegister(MCP_ADR_GPLAT, 0x00, pin_maske);

  // ADCON1-Register setzen (nur falls Analog-Pin)
  if((pin==MCP_GP0) || (pin==MCP_GP1) || (pin==MCP_GP2) || (pin==MCP_GP3)) {
    UB_MCP25050_writeRegister(MCP_ADR_ADCON1, reg_adcon1, pin_maske);
  }

  // event. ADC aktivieren (mit 2MHz)
  if(mode==MCP_ADC_IN) {
    P_MCP_initADC(0);
  }
}


//--------------------------------------------------------------
// GPIO-Pin auf Lo-Pegel setzen
// (Pin muss als Digital-Ausgang definiert sein)
//
// pin  : [MCP_GP0 bis MCP_GP6]
//--------------------------------------------------------------
void UB_MCP25050_GPIO_Lo(MCP_GPIO_t pin)
{
  uint8_t pin_maske=0x00;

  if(pin==MCP_GP0) pin_maske=0x01;
  if(pin==MCP_GP1) pin_maske=0x02;
  if(pin==MCP_GP2) pin_maske=0x04;
  if(pin==MCP_GP3) pin_maske=0x08;
  if(pin==MCP_GP4) pin_maske=0x10;
  if(pin==MCP_GP5) pin_maske=0x20;
  if(pin==MCP_GP6) pin_maske=0x40;
  if(pin==MCP_GP7) return; // GP7 geht nicht als Ausgang

  // GPLAT-Register setzen (auf Lo)
  UB_MCP25050_writeRegister(MCP_ADR_GPLAT, 0x00, pin_maske);
}


//--------------------------------------------------------------
// GPIO-Pin auf Hi-Pegel setzen
// (Pin muss als Digital-Ausgang definiert sein)
//
// pin  : [MCP_GP0 bis MCP_GP6]
//--------------------------------------------------------------
void UB_MCP25050_GPIO_Hi(MCP_GPIO_t pin)
{
  uint8_t pin_maske=0x00;

  if(pin==MCP_GP0) pin_maske=0x01;
  if(pin==MCP_GP1) pin_maske=0x02;
  if(pin==MCP_GP2) pin_maske=0x04;
  if(pin==MCP_GP3) pin_maske=0x08;
  if(pin==MCP_GP4) pin_maske=0x10;
  if(pin==MCP_GP5) pin_maske=0x20;
  if(pin==MCP_GP6) pin_maske=0x40;
  if(pin==MCP_GP7) return; // GP7 geht nicht als Ausgang

  // GPLAT-Register setzen (auf Hi)
  UB_MCP25050_writeRegister(MCP_ADR_GPLAT, 0xFF, pin_maske);
}


//--------------------------------------------------------------
// Pegel von GPIO Pin auslesen
// (Pin muss als Digital-Eingang definiert sein)
//
// pin  : [MCP_GP0 bis MCP_GP7]
// Return Wert :
//  -> bei Lo-Pegel = Bit_RESET
//  -> bei Hi-Pegel = Bit_SET
//--------------------------------------------------------------
BitAction UB_MCP25050_GPIO_Read(MCP_GPIO_t pin)
{
  BitAction ret_wert=Bit_RESET;
  uint8_t pin_maske=0x00;
  uint8_t wert;
  uint32_t n;
  CAN1_STATUS_t check;

  if(pin==MCP_GP0) pin_maske=0x01;
  if(pin==MCP_GP1) pin_maske=0x02;
  if(pin==MCP_GP2) pin_maske=0x04;
  if(pin==MCP_GP3) pin_maske=0x08;
  if(pin==MCP_GP4) pin_maske=0x10;
  if(pin==MCP_GP5) pin_maske=0x20;
  if(pin==MCP_GP6) pin_maske=0x40;
  if(pin==MCP_GP7) pin_maske=0x80; 


  //-------------------------------
  // A/D-Register auslesen
  // (8 Datenbytes werden erwartet)
  //-------------------------------
  tx_mcp25050.can_id=(MCP_RXF0|MCP_RD_AD);
  tx_mcp25050.anz_bytes=0x08;
  // STD-Remote-Frame senden
  UB_CAN1_send_std_remote(tx_mcp25050);

  // warten bis antwort erhalten
  for(n=0;n<MCP_WAIT4RECEIVE;n++) {
    check=UB_CAN1_receive(&rx_mcp25050);
    if(check==CAN1_RX_READY) {
      // wenn etwas empfangen wurde
      // test ob Antwort vom MCP25050
      if(rx_mcp25050.can_id==(MCP_RXF0|MCP_RD_AD)) {
        if(rx_mcp25050.frame_mode==CAN1_STD_DATA) {
          // Daten auslesen
          wert=rx_mcp25050.data[1]; // GPIO
          // gesuchtes Bit maskieren
          if((wert&pin_maske)!=0) {
            ret_wert=Bit_SET; 
          }
          else {
            ret_wert=Bit_RESET;
          }

          break;
        }
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion (schaltet ADC ein)
//
// VRef+ = Vcc
// VRef- = GND
// adc_speed [0...2] : 0=2 Mhz, 1=500 kHz, 2=125 kHz
//--------------------------------------------------------------
void P_MCP_initADC(uint8_t adc_speed)
{
  static uint8_t enable_ok=0;
  uint8_t reg_adcon1=0x00;

  // initialisierung darf nur einmal gemacht werden
  if(enable_ok!=0) {
    return;
  }

  if(adc_speed>2) adc_speed=2;

  reg_adcon1=0x00; // Referenz = Vcc und GND
  if(adc_speed==0) reg_adcon1|=0x00;  // 2 Mhz
  if(adc_speed==1) reg_adcon1|=0x40;  // 500 kHz
  if(adc_speed==2) reg_adcon1|=0x80;  // 125 kHz
  
  // ADCON1-Register setzen (Bit4...Bit7)
  UB_MCP25050_writeRegister(MCP_ADR_ADCON1, reg_adcon1, 0xF0);  

  // ADCON0-Register setzen (Bit7)
  UB_MCP25050_writeRegister(MCP_ADR_ADCON0, 0x80, 0x80);

  // init Mode speichern
  enable_ok=1;
}


//--------------------------------------------------------------
// startet ADC-Wandlung und liesst Ergebnis aus
// (Pin muss als Analog-Eingang definiert sein)
//
// pin         : [MCP_GP0 bis MCP_GP3]
// Return Wert : [0...1023]
//--------------------------------------------------------------
uint16_t UB_MCP25050_ADC_Read(MCP_GPIO_t pin)
{
  uint16_t ret_wert=0;
  uint32_t n;
  CAN1_STATUS_t check;
  uint8_t wert_lo, wert_hi;
 
  // GP4-GP7 gehen nicht als ADC  
  if((pin==MCP_GP4) || (pin==MCP_GP5)) return(0);
  if((pin==MCP_GP6) || (pin==MCP_GP7)) return(0);

  //-------------------------------
  // A/D-Register auslesen
  // (8 Datenbytes werden erwartet)
  //-------------------------------
  tx_mcp25050.can_id=(MCP_RXF0|MCP_RD_AD);
  tx_mcp25050.anz_bytes=0x08;
  // STD-Remote-Frame senden
  UB_CAN1_send_std_remote(tx_mcp25050);

  // warten bis antwort erhalten
  for(n=0;n<MCP_WAIT4RECEIVE;n++) {
    check=UB_CAN1_receive(&rx_mcp25050);
    if(check==CAN1_RX_READY) {
      // wenn etwas empfangen wurde
      // test ob Antwort vom MCP25050
      if(rx_mcp25050.can_id==(MCP_RXF0|MCP_RD_AD)) {
        if(rx_mcp25050.frame_mode==CAN1_STD_DATA) {
          // Daten auslesen
          if(pin==MCP_GP0) {
            wert_hi=rx_mcp25050.data[2]; // ADC0 Bit2-9
            wert_lo=rx_mcp25050.data[4]; // ADC0 Bit0+1
            ret_wert=(wert_hi<<2);
            if((wert_lo&0x04)!=0) ret_wert|=0x01;
            if((wert_lo&0x08)!=0) ret_wert|=0x02;
          }
          else if(pin==MCP_GP1) {
            wert_hi=rx_mcp25050.data[3]; // ADC1 Bit2-9
            wert_lo=rx_mcp25050.data[4]; // ADC1 Bit0+1
            ret_wert=(wert_hi<<2);
            if((wert_lo&0x40)!=0) ret_wert|=0x01;
            if((wert_lo&0x80)!=0) ret_wert|=0x02;
          }
          else if(pin==MCP_GP2) {
            wert_hi=rx_mcp25050.data[5]; // ADC2 Bit2-9
            wert_lo=rx_mcp25050.data[7]; // ADC2 Bit0+1
            ret_wert=(wert_hi<<2);
            if((wert_lo&0x04)!=0) ret_wert|=0x01;
            if((wert_lo&0x08)!=0) ret_wert|=0x02;
          }
          else if(pin==MCP_GP3) {
            wert_hi=rx_mcp25050.data[6]; // ADC3 Bit2-9
            wert_lo=rx_mcp25050.data[7]; // ADC3 Bit0+1
            ret_wert=(wert_hi<<2);
            if((wert_lo&0x40)!=0) ret_wert|=0x01;
            if((wert_lo&0x80)!=0) ret_wert|=0x02;
          }
          
          break;
        }
      }
    }
  }
    
  return(ret_wert);
}


//--------------------------------------------------------------
// schaltet PWM EIN
// (Pin muss als PWM-Ausgang definiert sein
//
// pin         : [MCP_GP2 bis MCP_GP3]
// pwm_speed [0...2]
//--------------------------------------------------------------
void UB_MCP25050_PWM_Enable(MCP_GPIO_t pin, uint8_t pwm_speed)
{
  uint8_t reg_tcon=0x00;

  // nur GP2 und GP3 gehen als PWM 
  if((pin!=MCP_GP2) && (pin!=MCP_GP3)) return;

  if(pwm_speed>2) pwm_speed=2;

  if(pin==MCP_GP2) {
    reg_tcon=0x80; // PWM enable
    if(pwm_speed==0) reg_tcon|=0x00;
    if(pwm_speed==1) reg_tcon|=0x10;
    if(pwm_speed==2) reg_tcon|=0x20;  

    // T1CON-Register setzen (Bit7,5,4,1,0)
    UB_MCP25050_writeRegister(MCP_ADR_T1CON, reg_tcon, 0xB3); 

    // PR1-Register setzen (auf 0xFF)
    UB_MCP25050_writeRegister(MCP_ADR_PR1, 0xFF, 0xFF);

    // PWM1-Register setzen (auf 0x80)
    UB_MCP25050_writeRegister(MCP_ADR_PWM1, 0x80, 0xFF);       
  }
  else if(pin==MCP_GP3) {
    reg_tcon=0x80; // PWM enable
    if(pwm_speed==0) reg_tcon|=0x00;
    if(pwm_speed==1) reg_tcon|=0x10;
    if(pwm_speed==2) reg_tcon|=0x20;  

    // T2CON-Register setzen (Bit7,5,4,1,0)
    UB_MCP25050_writeRegister(MCP_ADR_T2CON, reg_tcon, 0xB3); 

    // PR2-Register setzen (auf 0xFF)
    UB_MCP25050_writeRegister(MCP_ADR_PR2, 0xFF, 0xFF);

    // PWM2-Register setzen (auf 0x80)
    UB_MCP25050_writeRegister(MCP_ADR_PWM2, 0x80, 0xFF);       
  }
}


//--------------------------------------------------------------
// stellt Duty_Cycle eines PWM ein
// (Pin muss als PWM-Ausgang definiert sein
//  und PWM muss aktiviert sein)
//
// pin         : [MCP_GP2 bis MCP_GP3]
// duty_cycle  : [0...255]
//--------------------------------------------------------------
void UB_MCP25050_PWM_Value(MCP_GPIO_t pin, uint8_t duty_cycle)
{
  // nur GP2 und GP3 gehen als PWM
  if((pin!=MCP_GP2) && (pin!=MCP_GP3)) return;

  if(pin==MCP_GP2) {
    // PWM1-Register setzen
    UB_MCP25050_writeRegister(MCP_ADR_PWM1, duty_cycle, 0xFF);
  }
  else if(pin==MCP_GP3) {
    // PWM2-Register setzen
    UB_MCP25050_writeRegister(MCP_ADR_PWM2, duty_cycle, 0xFF);
  }
}


//--------------------------------------------------------------
// schreiben eines 8bit Wertes in ein beliebiges RAM-Register
// des MCP25050 (mit einer Maske)
//
// adr   : [0x1C...0x4F]
// wert  : [0x00...0xFF]
// maske : [0x00...0xFF] (Lo : Bit wird nicht veraendert)
//--------------------------------------------------------------
void UB_MCP25050_writeRegister(uint8_t adr, uint8_t wert, uint8_t maske)
{

  if(adr<0x1C) return;
  if(adr>0x4F) return;

  //------------------------
  // Register beschreiben
  // (3 Datenbytes)
  //------------------------
  tx_mcp25050.can_id=(MCP_RXF1|MCP_WR_REGISTER);
  tx_mcp25050.anz_bytes=0x03;
  tx_mcp25050.data[0]=adr;   // adresse
  tx_mcp25050.data[1]=maske; // maske
  tx_mcp25050.data[2]=wert;  // wert
  // STD-Daten-Frame senden
  UB_CAN1_send_std_data(tx_mcp25050);

  // kleine Pause (wichtig !!)
  P_MCP_delay(MCP_WAIT4BUSY);
}


//--------------------------------------------------------------
// interne Funktion
// pause fr Bearbeitungszeit im MCP25050
//--------------------------------------------------------------
void P_MCP_delay(uint32_t delay)
{
  uint32_t n;

  for(n=0;n<delay;n++) {
    // kleine Pause
  }
}
