//--------------------------------------------------------------
// File     : stm32_ub_mcp25050.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_MCP25050_H
#define __STM32F4_UB_MCP25050_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_can1.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"




//--------------------------------------------------------------
// CAN-IDs (8Bit MSB) vom MCP25050
// diese IDs werden durch das HEX-File im Eprom bestimmt
//--------------------------------------------------------------
#define  MCP_TXID0  0x06A0   // (OnBus-Message)
#define  MCP_RXF0   0x06C0   // (Remote-Frames)
#define  MCP_RXF1   0x0680   // (Data-Frames)



//--------------------------------------------------------------
// enum der 8 GPIO Pins
//--------------------------------------------------------------
typedef enum {
  MCP_GP0 = 0, // Pin1  (GPIO0, AN0)
  MCP_GP1,     // Pin2  (GPIO1, AN1)
  MCP_GP2,     // Pin3  (GPIO2, AN2, PWM1)
  MCP_GP3,     // Pin4  (GPIO3, AN3, PWM2)
  MCP_GP4,     // Pin5  (GPIO4)
  MCP_GP5,     // Pin6  (GPIO5)
  MCP_GP6,     // Pin10 (GPIO6)
  MCP_GP7      // Pin11 (GPI7) 
}MCP_GPIO_t;


//--------------------------------------------------------------
// enum vom GPIO-Mode
//--------------------------------------------------------------
typedef enum {
  MCP_GPIO_OUT = 0,  // Pin als Digital-Ausgang
  MCP_GPIO_IN,       // Pin als Digital-Eingang
  MCP_PWM_OUT,       // Pin als PWM-Ausgang
  MCP_ADC_IN         // Pin als Analog-Eingang
}MCP_PIN_MODE_t;



//--------------------------------------------------------------
// Remote-Frame-IDs (3Bit LSB) vom MCP25050
//--------------------------------------------------------------
#define  MCP_RD_AD        0x00  // read AD-Register      (8 Bytes)
#define  MCP_RD_CTRL      0x01  // read Control-Register (7 Bytes)
#define  MCP_RD_CONFIG    0x02  // read Config-Register  (5 Bytes)
#define  MCP_RD_CAN_ERR   0x03  // read CAN Error        (3 Bytes)
#define  MCP_RD_PWM       0x04  // read PWM Config       (6 Bytes)
#define  MCP_RD_USER1     0x05  // read User Memory-1    (8 Bytes)
#define  MCP_RD_USER2     0x06  // read User Memory-2    (8 Bytes)


//--------------------------------------------------------------
// Data-Frame-IDs (3Bit LSB) vom MCP25050
//--------------------------------------------------------------
#define  MCP_WR_REGISTER  0x00  // write Register        (3 Bytes)
#define  MCP_WR_TX_ID0    0x01  // write TX Message-ID 0 (4 Bytes)
#define  MCP_WR_TX_ID1    0x02  // write TX Message-ID 1 (4 Bytes)
#define  MCP_WR_TX_ID2    0x03  // write TX Message-ID 2 (4 Bytes)
#define  MCP_WR_IO_CONFIG 0x04  // write IO Config       (5 Bytes)
#define  MCP_WR_RX_MASK   0x05  // write RX Mask         (4 Bytes)
#define  MCP_WR_RXF0      0x06  // write RX Filter-0     (4 Bytes)
#define  MCP_WR_RXF1      0x07  // write RX Filter-1     (4 Bytes)



//--------------------------------------------------------------
// Register Adressen (im RAM vom MCP25050)
// (alle Adressen mit Offset von 0x1C)
//--------------------------------------------------------------
#define  MCP_ADR_IOINTEN  (0x00 + 0x1C)
#define  MCP_ADR_IOINTPO  (0x01 + 0x1C)
#define  MCP_ADR_GPLAT    (0x02 + 0x1C)
#define  MCP_ADR_GPDR     0x1F  // kein Offset
#define  MCP_ADR_T1CON    (0x05 + 0x1C)
#define  MCP_ADR_T2CON    (0x06 + 0x1C)
#define  MCP_ADR_PR1      (0x07 + 0x1C)
#define  MCP_ADR_PR2      (0x08 + 0x1C)
#define  MCP_ADR_PWM1     (0x09 + 0x1C)
#define  MCP_ADR_PWM2     (0x0A + 0x1C)
#define  MCP_ADR_ADCON0   (0x0E + 0x1C)
#define  MCP_ADR_ADCON1   (0x0F + 0x1C)
#define  MCP_ADR_STCON    (0x10 + 0x1C)
#define  MCP_ADR_OPTREG2  (0x11 + 0x1C)
#define  MCP_ADR_TXID1H   (0x24 + 0x1C)
#define  MCP_ADR_TXID1L   (0x25 + 0x1C)
#define  MCP_ADR_TXID2H   (0x28 + 0x1C)
#define  MCP_ADR_TXID2L   (0x29 + 0x1C)







//--------------------------------------------------------------
#define  MCP_WAIT4RECEIVE  500000  // warte bis receive
#define  MCP_WAIT4BUSY     500000  // warten bis bereit




//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
ErrorStatus UB_MCP25050_Init(void);
ErrorStatus UB_MCP25050_Check(void);
void UB_MCP25050_PIN_Define(MCP_GPIO_t pin, MCP_PIN_MODE_t mode);
void UB_MCP25050_GPIO_Lo(MCP_GPIO_t pin);
void UB_MCP25050_GPIO_Hi(MCP_GPIO_t pin);
BitAction UB_MCP25050_GPIO_Read(MCP_GPIO_t pin);
uint16_t UB_MCP25050_ADC_Read(MCP_GPIO_t pin);
void UB_MCP25050_PWM_Enable(MCP_GPIO_t pin, uint8_t pwm_speed); 
void UB_MCP25050_PWM_Value(MCP_GPIO_t pin, uint8_t duty_cycle);
void UB_MCP25050_writeRegister(uint8_t adr, uint8_t wert, uint8_t maske);



//--------------------------------------------------------------
#endif // __STM32F4_UB_MCP25050_H
